# -*- coding: utf-8 -*-
"""
All these form can use a custom django-crispy-forms layout from their layout setting
"""
from django import forms
from django.conf import settings
from django.contrib.auth.forms import AuthenticationForm
from django.contrib.auth.forms import PasswordResetForm as PasswordResetBaseForm
from django.contrib.auth.forms import SetPasswordForm as SetPasswordBaseForm
from django.utils.translation import ugettext

from registration.forms import RegistrationFormUniqueEmail

from captcha.fields import CaptchaField

from sveeaccounts.crispies import get_form_helper, default_helper

class PasswordResetForm(PasswordResetBaseForm):
    def __init__(self, *args, **kwargs):
        self.helper = default_helper(form_tag=False)
        helper = get_form_helper(getattr(settings, 'REGISTRATION_PASSWORD_RESET_HELPER', None), default=default_helper)
        if helper is not None:
            self.helper = helper()
        
        super(PasswordResetForm, self).__init__(*args, **kwargs)

class PasswordResetChangeForm(SetPasswordBaseForm):
    def __init__(self, *args, **kwargs):
        self.helper = default_helper(form_tag=False)
        helper = get_form_helper(getattr(settings, 'REGISTRATION_PASSWORD_RESET_CHANGE_HELPER', None), default=default_helper)
        if helper is not None:
            self.helper = helper()
        
        super(PasswordResetChangeForm, self).__init__(*args, **kwargs)


class RegistrationWithCaptchaForm(RegistrationFormUniqueEmail):
    captcha = CaptchaField()
    
    def __init__(self, *args, **kwargs):
        helper = get_form_helper(getattr(settings, 'REGISTRATION_FORM_HELPER', None), default=default_helper)
        if helper is not None:
            self.helper = helper()
        
        super(RegistrationWithCaptchaForm, self).__init__(*args, **kwargs)


class LoginForm(AuthenticationForm):
    
    def __init__(self, *args, **kwargs):
        helper = get_form_helper(getattr(settings, 'REGISTRATION_LOGIN_HELPER', None), default=default_helper)
        if helper is not None:
            self.helper = helper()
        
        super(LoginForm, self).__init__(*args, **kwargs)


class UserForm(forms.Form):
    """
    Form for user to change his infos
    """
    # Bind only some fields from the user model
    first_name = forms.CharField(label=ugettext('First name'), max_length=30, required=True)
    last_name = forms.CharField(label=ugettext("Last name"), max_length=30, required=True)
    email = forms.EmailField(label=ugettext("E-mail"), max_length=75, required=True)
    new_password1 = forms.CharField(label=ugettext("Password"), widget=forms.PasswordInput, required=False)
    new_password2 = forms.CharField(label=ugettext("Password (again)"), widget=forms.PasswordInput, required=False)
    
    def __init__(self, *args, **kwargs):
        self.instance = kwargs.pop('instance')
        
        helper = get_form_helper(getattr(settings, 'REGISTRATION_USER_EDIT_HELPER', None), default=default_helper)
        if helper is not None:
            self.helper = helper()
        
        super(UserForm, self).__init__(*args, **kwargs)

    def clean_new_password2(self):
        password1 = self.cleaned_data.get('new_password1')
        password2 = self.cleaned_data.get('new_password2')
        if password1 and password2:
            if password1 != password2:
                raise forms.ValidationError(ugettext("The two password fields didn't match."))
        return password2
    
    def save(self, *args, **kwargs):
        self.instance.first_name = self.cleaned_data["first_name"]
        self.instance.last_name = self.cleaned_data["last_name"]
        self.instance.email = self.cleaned_data["email"]
        self.instance.save()
        if self.cleaned_data.get("new_password2", None) and len(self.cleaned_data["new_password2"].strip())>0:
            self.instance.set_password(self.cleaned_data["new_password2"])
        
        return self.instance



# -*- coding: utf-8 -*-
"""
Stuff for form helpers with ``crispy_forms``

``crispy_forms`` import and usage is optionnal and no exception should be raised if 
you don't have installed it.
"""
import warnings

from django import forms
from django.conf import settings
from django.utils.importlib import import_module
from django.utils.translation import ugettext

def get_form_helper(helper_path, default=None):
    """
    Get the crispy_forms Helper from the given Python path
    
    @helper_path is a string containing a Python path to the wanted helper, @default is 
    a callable used as the default layout if the @helper_path import fails. @default 
    could be None if you don't want a default layout (and use the automated one 
    from crispy_forms).
    
    You should only use the None value for @default if you don't plan to use crispy_forms.
    
    Return a callable or None
    """
    if helper_path is None:
        return default
    
    dot = helper_path.rindex('.')
    module_name = helper_path[:dot]
    class_name = helper_path[dot + 1:]
    try:
        _class = getattr(import_module(module_name), class_name)
        return _class
    except (ImportError, AttributeError):
        warnings.warn('%s cannot be imported' % helper_path,
                      RuntimeWarning)
    return default

try:
    from crispy_forms.helper import FormHelper
    from crispy_forms.layout import Layout, Fieldset, Div, Submit
    from crispy_forms.bootstrap import FormActions
except ImportError:
    def default_helper():
        return None
    def UserProfileBaseHelper():
        return None
else:
    def default_helper(form_tag=True):
        helper = FormHelper()
        helper.form_action = '.'
        helper.form_tag = form_tag
        helper.form_style = 'inline'
        helper.add_input(Submit('submit', ugettext('Submit')))
        return helper



"""
URLconf for WithCaptchaBackend behaviors
"""
from django.conf import settings
from django.conf.urls.defaults import *
from django.views.generic.simple import direct_to_template
from django.contrib.auth import views as auth_views

from registration.views import activate
from registration.views import register

from sveeaccounts.forms import LoginForm, PasswordResetForm, PasswordResetChangeForm
from sveeaccounts.views import AccountUserForm

REGISTRATION_BLOCKED = getattr(settings, 'REGISTRATION_BLOCKED', False)
PASSWORD_RESET_BLOCKED = getattr(settings, 'PASSWORD_RESET_BLOCKED', False)
USER_PROFILE_BLOCKED = getattr(settings, 'USER_PROFILE_BLOCKED', False)

if REGISTRATION_BLOCKED:
    urlpatterns = patterns('',
        url(r'^activate/complete/$', direct_to_template, {'template': 'registration/blocked.html'}, name='registration_activation_complete'),
        url(r'^activate/(?P<activation_key>\w+)/$', direct_to_template, {'template': 'registration/blocked.html'}, name='registration_activate'),
        url(r'^register/$', direct_to_template, {'template': 'registration/blocked.html'}, name='registration_register'),
        url(r'^register/complete/$', direct_to_template, {'template': 'registration/blocked.html'}, name='registration_complete'),
        url(r'^register/closed/$', direct_to_template, {'template': 'registration/blocked.html'}, name='registration_disallowed'),
    )
else:
    urlpatterns = patterns('',
        url(r'^activate/complete/$', direct_to_template, {'template': 'registration/activation_complete.html'}, name='registration_activation_complete'),
        # Activation keys get matched by \w+ instead of the more specific
        # [a-fA-F0-9]{40} because a bad activation key should still get to the view;
        # that way it can return a sensible "invalid key" message instead of a
        # confusing 404.
        url(r'^activate/(?P<activation_key>\w+)/$', activate, {'backend': 'sveeaccounts.backends.WithCaptchaBackend'}, name='registration_activate'),
        # Register views
        url(r'^register/$', register, {'backend': 'sveeaccounts.backends.WithCaptchaBackend'}, name='registration_register'),
        url(r'^register/complete/$', direct_to_template, {'template': 'registration/registration_complete.html'}, name='registration_complete'),
        url(r'^register/closed/$', direct_to_template, {'template': 'registration/registration_closed.html'}, name='registration_disallowed'),
    )

urlpatterns += patterns('',
    # Login/logout views
    url(r'^login/$',
        auth_views.login,
        {'template_name': 'registration/login.html', 'authentication_form': LoginForm},
        name='auth_login'),
    url(r'^logout/$',
        auth_views.logout_then_login,
        {'login_url': '/'},
        name='auth_logout'),
)

if not USER_PROFILE_BLOCKED:
    urlpatterns += patterns('',
        # User profile form
        url(r'^profile/$',
            AccountUserForm.as_view(),
            name='auth_user_form'),
    )

if not PASSWORD_RESET_BLOCKED:
    urlpatterns += patterns('',
        # Password reset views for anonymous (copy paste from contrib.auth)
        url(r'^password/$', auth_views.password_reset, {'password_reset_form': PasswordResetForm}, name='auth_password_reset'),
        url(r'^password/reset/done/$', 'django.contrib.auth.views.password_reset_done', name='password_reset_done'),
        url(r'^password/reset/(?P<uidb36>[0-9A-Za-z]+)-(?P<token>.+)/$', auth_views.password_reset_confirm, {'set_password_form': PasswordResetChangeForm}, name='auth_password_reset_confirm'),
        url(r'^password/reset/complete/$', 'django.contrib.auth.views.password_reset_complete', name='password_reset_complete'),
    )



# -*- coding: utf-8 -*-
"""
Accounts views
"""
from django.views.generic import UpdateView
from django.core.urlresolvers import reverse

from django.contrib.auth.models import User

from braces.views import LoginRequiredMixin

from sveeaccounts.forms import UserForm

class AccountUserForm(LoginRequiredMixin, UpdateView):
    """
    The form view to edit the user account and his profile
    
    The following attributes are required and you have also to define a 
    'get_success_url' method.
    """
    template_name = "registration/user_form.html"
    form_class = UserForm
    model = User
    
    def get_object(self, *args, **kwargs):
        return self.request.user
    
    def get_success_url(self):
        return reverse('auth_user_form')
    
    def get_initial(self):
        """
        Returns the initial data to use for forms on this view.
        """
        u = self.get_object()
        return {
            'first_name': u.first_name,
            'last_name': u.last_name,
            'email': u.email,
        }

